@@ -22,6 +22,7 @@ import logging
 import os
 from pprint import pformat
 import re
+from six.moves.urllib.parse import quote
 import time
 import xml.etree.ElementTree as XML
 
@@ -78,9 +79,17 @@ class JenkinsManager(object):
             output_dir = os.path.join(
                 output_dir, os.path.basename(item))
             output_fn = os.path.join(output_dir, 'config.xml')
+        else:
+            logger.warn('(Deprecated) The default output behavior of'
+                        ' `jenkins-jobs test` when given the --output'
+                        ' flag will change in JJB 3.0.'
+                        ' Instead of writing jobs to OUTPUT/jobname;'
+                        ' they will be written to OUTPUT/jobname/config.xml.'
+                        ' The new behavior can be enabled by the passing'
+                        ' `--config-xml` parameter.')
 
         if output_dir != output:
-            logger.info("Creating directory %s" % output_dir)
+            logger.debug("Creating directory %s" % output_dir)
             try:
                 os.makedirs(output_dir)
             except OSError:
@@ -108,10 +117,21 @@ class JenkinsManager(object):
         # returns job name or url based on config option
         if self._jjb_config.builder['print_job_urls']:
             return self._jjb_config.jenkins['url'] + \
-                '/job/' + '/job/'.join(job_name.split('/'))
+                '/job/' + quote(
+                    '/job/'.join(job_name.split('/')).encode('utf8')) + '/'
         else:
             return job_name
 
+    def _view_format(self, view_name):
+        # returns job name or url based on config option
+        if self._jjb_config.builder['print_job_urls']:
+            parts = view_name.split('/')
+            return self._jjb_config.jenkins['url'] + \
+                ''.join(['/job/' + item for item in parts[:-1]]) + \
+                '/view/' + parts[-1] + '/'
+        else:
+            return view_name
+
     def update_job(self, job_name, xml):
         if self.is_job(job_name):
             logger.info("Reconfiguring jenkins job {0}".format(
@@ -122,12 +142,11 @@ class JenkinsManager(object):
                 self._job_format(job_name)))
             self.jenkins.create_job(job_name, xml)
 
-    def is_job(self, job_name):
-        # first use cache
-        if job_name in self.job_list:
-            return True
+    def is_job(self, job_name, use_cache=True):
+        if use_cache:
+            if job_name in self.job_list:
+                return True
 
-        # if not exists, use jenkins
         return self.jenkins.job_exists(job_name)
 
     def get_job_md5(self, job_name):
@@ -190,7 +209,9 @@ class JenkinsManager(object):
             keep = []
         for job in jobs:
             # python-jenkins stores the folder and name as 'fullname'
-            if job['fullname'] not in keep:
+            # Check if the job was deleted when his parent folder was deleted
+            if job['fullname'] not in keep and \
+                    self.is_job(job['fullname'], use_cache=False):
                 if self.is_managed(job['fullname']):
                     logger.info("Removing obsolete jenkins job {0}"
                                 .format(job['fullname']))
@@ -230,8 +251,14 @@ class JenkinsManager(object):
             logger.debug("'{0}' has not changed".format(job.name))
         return changed
 
+    def exists(self, job):
+        exists = self.jenkins.job_exists(job.name)
+        if not exists:
+            logger.debug("'{0}' does not currently exist".format(job.name))
+        return exists
+
     def update_jobs(self, xml_jobs, output=None, n_workers=None,
-                    config_xml=False):
+                    existing_only=None, config_xml=False):
         orig = time.time()
 
         logger.info("Number of jobs generated:  %d", len(xml_jobs))
@@ -239,7 +266,7 @@ class JenkinsManager(object):
 
         if (output and not hasattr(output, 'write') and
                 not os.path.isdir(output)):
-            logger.info("Creating directory %s" % output)
+            logger.debug("Creating directory %s" % output)
             try:
                 os.makedirs(output)
             except OSError:
@@ -283,6 +310,16 @@ class JenkinsManager(object):
         logging.debug("Filtered for changed jobs in %ss",
                       (time.time() - step))
 
+        if existing_only:
+            # Filter out the jobs not already in the cache
+            logging.debug('Filtering %d jobs for existing jobs',
+                          len(jobs))
+            step = time.time()
+            jobs = [job for job in jobs
+                    if self.exists(job)]
+            logging.debug("Filtered for existing jobs in %ss",
+                          (time.time() - step))
+
         if not jobs:
             return [], 0
 
@@ -376,14 +413,16 @@ class JenkinsManager(object):
 
     def update_view(self, view_name, xml):
         if self.is_view(view_name):
-            logger.info("Reconfiguring jenkins view {0}".format(view_name))
+            logger.info("Reconfiguring jenkins view {0}".format(
+                self._view_format(view_name)))
             self.jenkins.reconfig_view(view_name, xml)
         else:
-            logger.info("Creating jenkins view {0}".format(view_name))
+            logger.info("Creating jenkins view {0}".format(
+                self._view_format(view_name)))
             self.jenkins.create_view(view_name, xml)
 
     def update_views(self, xml_views, output=None, n_workers=None,
-                     config_xml=False):
+                     existing_only=None, config_xml=False):
         orig = time.time()
 
         logger.info("Number of views generated:  %d", len(xml_views))
@@ -426,6 +465,16 @@ class JenkinsManager(object):
         logging.debug("Filtered for changed views in %ss",
                       (time.time() - step))
 
+        if existing_only:
+            # Filter out the jobs not already in the cache
+            logging.debug('Filtering %d views for existing jobs',
+                          len(views))
+            step = time.time()
+            views = [view for view in views
+                    if self.exists(view)]
+            logging.debug("Filtered for existing views in %ss",
+                          (time.time() - step))
+
         if not views:
             return [], 0
 
