@@ -41,6 +41,7 @@ from jenkins_jobs.modules import project_externaljob
 from jenkins_jobs.modules import project_flow
 from jenkins_jobs.modules import project_matrix
 from jenkins_jobs.modules import project_maven
+from jenkins_jobs.modules import project_multibranch
 from jenkins_jobs.modules import project_multijob
 from jenkins_jobs.modules import view_list
 from jenkins_jobs.modules import view_pipeline
@@ -188,6 +189,10 @@ class BaseScenariosTestCase(testscenarios.TestWithScenarios, BaseTestCase):
                 project = project_flow.Flow(registry)
             elif (yaml_content['project-type'] == "multijob"):
                 project = project_multijob.MultiJob(registry)
+            elif (yaml_content['project-type'] == "multibranch"):
+                project = project_multibranch.WorkflowMultiBranch(registry)
+            elif (yaml_content['project-type'] == "multibranch-defaults"):
+                project = project_multibranch.WorkflowMultiBranchDefaults(registry)  # noqa
             elif (yaml_content['project-type'] == "externaljob"):
                 project = project_externaljob.ExternalJob(registry)
 
@@ -241,7 +246,8 @@ class SingleJobTestCase(BaseScenariosTestCase):
     def test_yaml_snippet(self):
         config = self._get_config()
 
-        expected_xml = self._read_utf8_content()
+        expected_xml = self._read_utf8_content().strip() \
+            .replace('<BLANKLINE>', '').replace('\n\n', '\n')
 
         parser = YamlParser(config)
         parser.parse(self.in_filename)
@@ -285,14 +291,14 @@ class SingleJobTestCase(BaseScenariosTestCase):
 
         # Prettify generated XML
         pretty_xml = u"\n".join(job.output().decode('utf-8')
-                                for job in xml_jobs)
+                                for job in xml_jobs) \
+            .strip().replace('\n\n', '\n')
 
         self.assertThat(
             pretty_xml,
             testtools.matchers.DocTestMatches(expected_xml,
-                                              doctest.ELLIPSIS |
-                                              doctest.REPORT_NDIFF)
-        )
+                                            doctest.ELLIPSIS |
+                                            doctest.REPORT_NDIFF))
 
 
 class JsonTestCase(BaseScenariosTestCase):
